@@ -1,6 +1,7 @@
-# pValueNonNested: Find p value for a nonnested model comparison
+### pValueNonNested: Find p value for a nonnested model comparison
+### Last updated: 3 June 2018
 
-pValueNonNested <- function(outMod1, outMod2, dat1Mod1, dat1Mod2, dat2Mod1, dat2Mod2, 
+pValueNonNested <- function(outMod1, outMod2, dat1Mod1, dat1Mod2, dat2Mod1, dat2Mod2,
     usedFit = NULL, nVal = NULL, pmMCARval = NULL, pmMARval = NULL, df = 0, onetailed = FALSE) {
     mod1 <- clean(dat1Mod1, dat1Mod2)
     dat1Mod1 <- mod1[[1]]
@@ -10,68 +11,72 @@ pValueNonNested <- function(outMod1, outMod2, dat1Mod1, dat1Mod2, dat2Mod1, dat2
     dat2Mod2 <- mod2[[2]]
 	usedFit <- cleanUsedFit(usedFit, colnames(dat1Mod1@fit), colnames(dat1Mod2@fit), colnames(dat2Mod1@fit), colnames(dat2Mod2@fit))
     revDirec <- (usedFit %in% getKeywords()$reversedFit)  # CFA --> FALSE, RMSEA --> TRUE
-    
-    if (!isTRUE(all.equal(unique(dat2Mod1@paramValue), unique(dat2Mod2@paramValue)))) 
+
+    if (!isTRUE(all.equal(unique(dat2Mod1@paramValue), unique(dat2Mod2@paramValue))))
         stop("'dat2Mod1' and 'dat2Mod2' are based on different data and cannot be compared, check your random seed")
-    if (!isTRUE(all.equal(unique(dat1Mod1@paramValue), unique(dat1Mod2@paramValue)))) 
+    if (!isTRUE(all.equal(unique(dat1Mod1@paramValue), unique(dat1Mod2@paramValue))))
         stop("'dat1Mod1' and 'dat1Mod2' are based on different data and cannot be compared, check your random seed")
-    if (!multipleAllEqual(unique(dat2Mod1@n), unique(dat2Mod2@n), unique(dat1Mod1@n), 
-        unique(dat1Mod2@n))) 
+    if (!multipleAllEqual(unique(dat2Mod1@n), unique(dat2Mod2@n), unique(dat1Mod1@n),
+        unique(dat1Mod2@n)))
         stop("Models are based on different values of sample sizes")
-    if (!multipleAllEqual(unique(dat2Mod1@pmMCAR), unique(dat2Mod2@pmMCAR), unique(dat1Mod1@pmMCAR), 
-        unique(dat1Mod2@pmMCAR))) 
+    if (!multipleAllEqual(unique(dat2Mod1@pmMCAR), unique(dat2Mod2@pmMCAR), unique(dat1Mod1@pmMCAR),
+        unique(dat1Mod2@pmMCAR)))
         stop("Models are based on different values of the percent completely missing at random")
-    if (!multipleAllEqual(unique(dat2Mod1@pmMAR), unique(dat2Mod2@pmMAR), unique(dat1Mod1@pmMAR), 
-        unique(dat1Mod2@pmMAR))) 
+    if (!multipleAllEqual(unique(dat2Mod1@pmMAR), unique(dat2Mod2@pmMAR), unique(dat1Mod1@pmMAR),
+        unique(dat1Mod2@pmMAR)))
         stop("Models are based on different values of the percent missing at random")
-    
-    if (is.null(nVal) || is.na(nVal)) 
+
+    if (is.null(nVal) || is.na(nVal))
         nVal <- NULL
-    if (is.null(pmMCARval) || is.na(pmMCARval)) 
+    if (is.null(pmMCARval) || is.na(pmMCARval))
         pmMCARval <- NULL
-    if (is.null(pmMARval) || is.na(pmMARval)) 
+    if (is.null(pmMARval) || is.na(pmMARval))
         pmMARval <- NULL
-    
-    condition <- c(length(unique(dat2Mod1@pmMCAR)) > 1, length(unique(dat2Mod1@pmMAR)) > 
+
+    condition <- c(length(unique(dat2Mod1@pmMCAR)) > 1, length(unique(dat2Mod1@pmMAR)) >
         1, length(unique(dat2Mod1@n)) > 1)
     condValue <- cbind(dat2Mod1@pmMCAR, dat2Mod1@pmMAR, dat2Mod1@n)
     colnames(condValue) <- c("Percent MCAR", "Percent MAR", "N")
     condValue <- condValue[, condition]
-    if (is.null(condValue) || length(condValue) == 0) 
+    if (is.null(condValue) || length(condValue) == 0)
         condValue <- NULL
     predictorVal <- rep(NA, 3)
     if (condition[3]) {
-        ifelse(is.null(nVal), stop("Please specify the sample size value, 'nVal', because the sample size in the result object is varying"), 
+        ifelse(is.null(nVal), stop("Please specify the sample size value, 'nVal', because the sample size in the result object is varying"),
             predictorVal[3] <- nVal)
     }
     if (condition[1]) {
-        ifelse(is.null(pmMCARval), stop("Please specify the percent of completely missing at random, 'pmMCARval', because the percent of completely missing at random in the result object is varying"), 
+        ifelse(is.null(pmMCARval), stop("Please specify the percent of completely missing at random, 'pmMCARval', because the percent of completely missing at random in the result object is varying"),
             predictorVal[1] <- pmMCARval)
     }
     if (condition[2]) {
-        ifelse(is.null(pmMARval), stop("Please specify the percent of missing at random, 'pmMARval', because the percent of missing at random in the result object is varying"), 
+        ifelse(is.null(pmMARval), stop("Please specify the percent of missing at random, 'pmMARval', because the percent of missing at random in the result object is varying"),
             predictorVal[2] <- pmMARval)
     }
     predictorVal <- predictorVal[condition]
-    
+
     Data1 <- as.data.frame((dat1Mod1@fit - dat1Mod2@fit)[, usedFit])
     Data2 <- as.data.frame((dat2Mod1@fit - dat2Mod2@fit)[, usedFit])
-	
+
     if(is(outMod1, "MxModel") & is(outMod2, "MxModel")) {
-		cutoff <- semTools::fitMeasuresMx(outMod1)[usedFit] - semTools::fitMeasuresMx(outMod2)[usedFit]
+		cutoff <- fitMeasuresMx(outMod1)[usedFit] - fitMeasuresMx(outMod2)[usedFit]
 	} else if (is(outMod1, "lavaan") & is(outMod2, "lavaan")) {
-		cutoff <- inspect(outMod1, "fit")[usedFit] - inspect(outMod2, "fit")[usedFit]
+	  cutoff <- lavaan::fitMeasures(outMod1, fit.measures = usedFit) -
+	            lavaan::fitMeasures(outMod2, fit.measures = usedFit)
+	} else if (is(outMod1, "lavaan.mi") & is(outMod2, "lavaan.mi")) {
+	  cutoff <- getMethod("fitMeasures", "lavaan.mi")(outMod1, fit.measures = usedFit) -
+	            getMethod("fitMeasures", "lavaan.mi")(outMod2, fit.measures = usedFit)
 	} else {
-		stop("The 'outMod1' and 'outMod2' arguments must be both lavaan objects or MxModel objects.")
+		stop("The 'outMod1' and 'outMod2' arguments must be both lavaan(.mi) objects or MxModel objects.")
 	}
-    
+
     result1 <- NULL
     result2 <- NULL
     if (any(condition)) {
-        result1 <- pValueDataFrame(cutoff, Data1, revDirec, x = condValue, xval = predictorVal, 
+        result1 <- pValueDataFrame(cutoff, Data1, revDirec, x = condValue, xval = predictorVal,
             df = df, asLogical = FALSE)
         names(result1) <- usedFit
-        result2 <- pValueDataFrame(cutoff, Data2, !revDirec, x = condValue, xval = predictorVal, 
+        result2 <- pValueDataFrame(cutoff, Data2, !revDirec, x = condValue, xval = predictorVal,
             df = df, asLogical = FALSE)
         names(result2) <- usedFit
     } else {
@@ -103,4 +108,4 @@ pValueNonNested <- function(outMod1, outMod2, dat1Mod1, dat1Mod2, dat2Mod1, dat2
 
 twoTailedPValue <- function(vec) {
     apply(cbind(vec, 1 - vec), 1, min) * 2
-} 
+}
